@@ -1,6 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+__author__ = 'Scott Burns <scott.s.burns@vanderbilt.edu>'
+__license__ = 'MIT'
+__copyright__ = '2014, Vanderbilt University'
+
 """
 This module exposes the REDCap API through the Project class. Instantiate the
 class with the URL to your REDCap system along with an API key, probably
@@ -13,6 +17,8 @@ in conjunction with your Project to filter your data.
 """
 
 from .project import Project
+import warnings
+warnings.warn("Query & QueryGroup will be removed at 1.0.", DeprecationWarning)
 from .query import Query, QueryGroup
 from .request import RCRequest, RCAPIError, RedcapError
-from .version import VERSION as __version__
\ No newline at end of file
+from .version import VERSION as __version__
