@@ -13,13 +13,16 @@ except ImportError:
 class ProjectTests(unittest.TestCase):
     """docstring for ProjectTests"""
 
-    def setUp(self):
-        self.url = 'https://redcap.vanderbilt.edu/api/'
-        self.long_proj = Project(self.url, '1387872621BBF1C17CC47FD8AE25FF54')
-        self.reg_proj = Project(self.url, '8E66DB6844D58E990075AFB51658A002')
-        self.ssl_proj = Project(self.url, '8E66DB6844D58E990075AFB51658A002',
-            verify_ssl=False)
+    url = 'https://redcap.vanderbilt.edu/api/'
+    bad_url = 'https://redcap.vanderbilt.edu/api'
+    reg_token = '8E66DB6844D58E990075AFB51658A002'
+    long_proj = Project(url, '1387872621BBF1C17CC47FD8AE25FF54')
+    reg_proj = Project(url, reg_token)
+    ssl_proj = Project(url, reg_token, verify_ssl=False)
+    survey_proj = Project(url, '37CAB1ABC2FEB3BB9D821DF13BA38A7B')
 
+    def setUp(self):
+        pass
     def tearDown(self):
         pass
 
@@ -69,6 +72,22 @@ class ProjectTests(unittest.TestCase):
         for record in data:
             self.assertIsInstance(record, dict)
 
+    def test_import_records(self):
+        "Test record import"
+        data = self.reg_proj.export_records()
+        response = self.reg_proj.import_records(data)
+        self.assertIn('count', response)
+        self.assertNotIn('error', response)
+
+    def test_import_exception(self):
+        "Test record import throws RedcapError for bad import"
+        data = self.reg_proj.export_records()
+        data[0]['non_existent_key'] = 'foo'
+        with self.assertRaises(RedcapError) as cm:
+            self.reg_proj.import_records(data)
+        exc = cm.exception
+        self.assertIn('error', exc.args[0])
+
     def is_good_csv(self, csv_string):
         "Helper to test csv strings"
         return isinstance(csv_string, basestring)
@@ -83,6 +102,13 @@ class ProjectTests(unittest.TestCase):
         csv = self.reg_proj.export_metadata(format='csv')
         self.assertTrue(self.is_good_csv(csv))
 
+    def test_bad_creds(self):
+        "Test that exceptions are raised with bad URL or tokens"
+        with self.assertRaises(RedcapError):
+            Project(self.bad_url, self.reg_token)
+        with self.assertRaises(RedcapError):
+            Project(self.url, '1')
+
     def test_fem_export(self):
         """ Test fem export in obj format gives list of dicts"""
         fem = self.long_proj.export_fem(format='obj')
@@ -170,6 +196,33 @@ class ProjectTests(unittest.TestCase):
         self.assertIn('verify', post_kwargs)
         self.assertTrue(post_kwargs['verify'])
 
+    def test_export_data_access_groups(self):
+        """Test we get 'redcap_data_access_group' in exported data"""
+        records = self.reg_proj.export_records(export_data_access_groups=True)
+        for record in records:
+            self.assertIn('redcap_data_access_group', record)
+        # When not passed, that key shouldn't be there
+        records = self.reg_proj.export_records()
+        for record in records:
+            self.assertNotIn('redcap_data_access_group', record)
+
+    def test_export_survey_fields(self):
+        """Test that we get the appropriate survey keys in the exported
+        data.
+
+        Note that the 'demographics' form has been setup as the survey
+        in the `survey_proj` project. The _timestamp field will vary for
+        users as their survey form will be named differently"""
+        records = self.survey_proj.export_records(export_survey_fields=True)
+        for record in records:
+            self.assertIn('redcap_survey_identifier', record)
+            self.assertIn('demographics_timestamp', record)
+        # The regular project doesn't have a survey setup. Users should
+        # be able this argument as True but it winds up a no-op.
+        records = self.reg_proj.export_records(export_survey_fields=True)
+        for record in records:
+            self.assertNotIn('redcap_survey_identifier', record)
+            self.assertNotIn('demographics_timestamp', record)
 
     @unittest.skipIf(skip_pd, "Couldn't import pandas")
     def test_metadata_to_df(self):
@@ -215,3 +268,24 @@ class ProjectTests(unittest.TestCase):
         response = self.long_proj.import_records(long_df)
         self.assertIn('count', response)
         self.assertNotIn('error', response)
+
+    def test_date_formatting(self):
+        """Test date_format parameter"""
+
+        def import_factory(date_string):
+            return [{'study_id': '1',
+                     'dob': date_string}]
+
+        # Default YMD with dashes
+        import_ymd = import_factory('2000-01-01')
+        response = self.reg_proj.import_records(import_ymd)
+        self.assertEqual(response['count'], 1)
+
+        # DMY with /
+        import_dmy = import_factory('31/01/2000')
+        response = self.reg_proj.import_records(import_dmy, date_format='DMY')
+        self.assertEqual(response['count'], 1)
+
+        import_mdy = import_factory('12/31/2000')
+        response = self.reg_proj.import_records(import_mdy, date_format='MDY')
+        self.assertEqual(response['count'], 1)
